@@ -11,7 +11,7 @@
 #'
 #' The `coeff` parameter must be either a character naming the coefficient to
 #' compute, or a function. If a character, it must be one of the following
-#' names: "kinship", "phi", "phi11", "k00", "k01", "k02", "k10", "k11", "k12",
+#' names: "inb", "kinship", "phi", "phi11", "k00", "k01", "k02", "k10", "k11", "k12",
 #' "k20", "k21" or "k22".
 #'
 #' If `coeff` is a function, it must take three arguments named `ped`, `ids` and
@@ -24,8 +24,12 @@
 #' @param peds A list of lists. See details.
 #' @param coeff A string identifying which coefficient to compute. See Details
 #'   for legal values.
-#' @param xlab,ylab,col,lty Plotting parameters
-#' @param ... Further parameters passed on to [matplot()]
+#' @param rseq A numeric vector of recombination rates. By default `seq(from =
+#'   0, by = 0.5, length = 11)`.
+#' @param xlab,ylab Axis labels.
+#' @param col,lty,lwd Plotting parameters.
+#' @param ... Further parameters passed on to [matplot()].
+#'
 #' @return NULL
 #'
 #' @examples
@@ -95,12 +99,16 @@
 #'
 #' @importFrom graphics legend matplot
 #' @export
-twoLocusPlot = function(peds, coeff = "k11", xlab = "Recombination rate",
-                        ylab = NA, col = seq_along(peds), lty = 1, ...) {
+twoLocusPlot = function(peds, coeff = "k11", rseq = seq(0, 0.5, length = 11),
+                        xlab = "Recombination rate",
+                        ylab = NA, col = seq_along(peds), lty = 1, lwd = 1, ...) {
 
   if(!is.list(peds))
     stop2("Argument `peds` must be a list")
 
+  if(is.ped(peds[[1]]))
+    peds = list(peds)
+
   for(p in peds) {
     if(!is.list(p) || !setequal(names(p), c("ped", "ids")))
       stop2("Each entry of `peds` must be a list with names 'ped' and 'ids'")
@@ -108,24 +116,30 @@ twoLocusPlot = function(peds, coeff = "k11", xlab = "Recombination rate",
       stop2("Something is wrong with the input")
   }
 
-  rseq = seq(0, 0.5, length = 11)  # recombination values
+  if(length(coeff) != 1)
+    stop2("Argument `coeff` must have length 1: ", coeff)
 
   # Choose function
   if(is.function(coeff)) {
     myFUN = coeff
     if(is.na(ylab)) ylab = as.character(substitute(coeff))
   }
-  else if(is.character(coeff) && length(coeff) == 1) {
-    if(coeff %in% c("kinship", "phi", "phi11")) {
-      myFUN = function(ped, ids, rho) twoLocusKinship(ped, ids, rho = rho)
-      if(is.na(ylab)) ylab = "Two locus kinship"
-    }
-    else {
-      myFUN = function(ped, ids, rho) twoLocusIBD(ped, ids, rho = rho, coefs = coeff)
-      if(is.na(ylab)) ylab = coeff
-    }
-  }
-  else stop2("Illegal value of `coeff`: ", coeff)
+  else
+    switch(coeff,
+      inb = {
+        myFUN = function(ped, ids, rho) twoLocusInbreeding(ped, id = ids, rho = rho)
+        if(is.na(ylab)) ylab = "Two locus inbreeding"
+      },
+      kinship =, phi =, phi11 = {
+        myFUN = function(ped, ids, rho) twoLocusKinship(ped, ids, rho = rho)
+        if(is.na(ylab)) ylab = "Two locus kinship"
+      },
+      k00 =, k01 =, k02 =, k10 =, k11 =, k12 =, k20 =, k21 =, k22 = {
+        myFUN = function(ped, ids, rho) twoLocusIBD(ped, ids, rho = rho, coefs = coeff)
+        if(is.na(ylab)) ylab = coeff
+      },
+      stop2("Illegal value of `coeff`: ", coeff)
+    )
 
   # Compute coefficients
   kvals = sapply(peds, function(x)
@@ -133,6 +147,8 @@ twoLocusPlot = function(peds, coeff = "k11", xlab = "Recombination rate",
 
   # Plot
   matplot(rseq, kvals, type = "l", xlab = xlab,
-          ylab = ylab, col = col, lty = lty, ...)
-  legend("topright", names(peds), col = col, lty = lty)
+          ylab = ylab, col = col, lty = lty, lwd = lwd, ...)
+
+  if(!is.null(names(peds)))
+     legend("topright", names(peds), col = col, lty = lty, lwd = lwd)
 }
